import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * This is the class that places the various elements of Pacman on a game board.
 * 
 * @author Deborah A. Trytten
 * @version 1.0
 */
public class GameBoard extends World
{
    // This is the size of the game board
    private static final int ROW_SIZE=15;
    private static final int COL_SIZE=13;
    
    // We have to have these constants so that Pacman knows where to go to restart
    // These numbers have to match the matrix below (PACMAN_INITIAL)
    // If the board is changed, these will probably have to be too
    private static final int PACMAN_START_ROW=11;
    private static final int PACMAN_START_COL=6;
 
    // Here are the things that are on the board
    private static final int PATH=0;
    private static final int WALL=1;
    private static final int DOT=2;
    private static final int GHOST_PEN=3;
    private static final int ENERGIZER=4;
    private static final int TUNNEL=5;
    private static final int PACMAN_INITIAL=6;

    // This array must be ROW_SIZE by COL_SIZE or all hell will break loose in the code
     private static final int[][] board={ 
                                    {0,0,0,0,0,0,0,0,0,0,0,0,0},
                                    {0,0,0,0,0,0,0,0,0,0,0,0,0},
                                    {1,1,1,1,1,1,1,1,1,1,1,1,1},
                                    {1,4,0,2,0,2,0,2,0,2,0,4,1},
                                    {1,0,1,1,1,1,0,1,1,1,1,0,1},
                                    {1,2,1,1,1,1,0,1,1,1,1,2,1},
                                    {1,0,1,1,0,0,0,0,0,1,1,0,1},
                                    {1,2,1,1,0,0,0,0,0,1,1,2,1},
                                    {5,0,0,0,0,3,3,3,0,0,0,0,5},
                                    {1,2,1,1,0,0,0,0,0,1,1,2,1},
                                    {1,0,1,1,0,0,0,0,0,1,1,0,1},
                                    {1,2,1,1,1,1,6,1,1,1,1,2,1},
                                    {1,0,1,1,1,1,0,1,1,1,1,0,1},
                                    {1,4,0,2,0,2,0,2,0,2,0,4,1},
                                    {1,1,1,1,1,1,1,1,1,1,1,1,1}
                                };
     
     
    public GameBoard()
    {    
         // Create a new world with 20x20 cells with a cell size of 10x10 pixels.
        super(COL_SIZE, ROW_SIZE, 20);
        
        // Make empty cells black squares
        setBackground("../images/black.gif"); 
        
        // Keep track of the number of ghosts so we can make them change color
        int ghostCount=0;
       
        for (int i=0; i<COL_SIZE; i++)
        {
            for (int j=0; j<ROW_SIZE; j++)
            {
                  
                switch (board[j][i]) // transposing the indices makes the images appear right side up
                  {
                      case PATH:
                                break;
                      case WALL: // add a Wall
                                addObject(new Wall(), i,j);
                                break;
                      case DOT: // add a Dot
                                addObject(new Dot(),i,j);
                                break;
                      case GHOST_PEN: // add the Ghost
                                addObject(new Ghost(ghostCount),i,j);
                                ghostCount++;
                                break;
                      case ENERGIZER: // add an energizer
                                addObject(new Energizer(),i,j);
                                break;
                      case TUNNEL: // add a tunnel
                                addObject(new Tunnel(),i,j);
                                break;
                      case PACMAN_INITIAL: // start pacman
                                addObject(new PacMan(),i,j);
                                break;
                                
                      default:
                            System.out.println("We shouldn't be here");
                            break;
                    }// end switch
    
              } //end for j
         }// end for i
         
    // Tell the word that PacMan should be painted on top of Ghost, which should be painted on 
    // top of everything else.  You can only order the classes, not the objects.
    setPaintOrder(PacMan.class, Ghost.class);
    } // end constructor
    
   
} // end GameBoard
