import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.util.List;

/**
 * This class implements the ghosts that eat PacMan.
 * 
 * @author Deborah Trytten modified by DFH
 */
public class Ghost extends Actor
{
    private static final int NUM_GHOSTS = 4;
    
    // Each ghost has a different image because they are a different color
    // Be incredibly careful with the capitalization of the image files--it is case sensitive, including the extension
    private static final GreenfootImage TURQUOISE_GHOST = new GreenfootImage("../images/turquoiseghost.gif");
    private static final GreenfootImage ORANGE_GHOST = new GreenfootImage("../images/orangeghost.gif");
    private static final GreenfootImage RED_GHOST = new GreenfootImage("../images/redghost.gif");
    private static final GreenfootImage PINK_GHOST = new GreenfootImage("../images/pinkghost.gif");
    private static final GreenfootImage VULNERABLE_GHOST = new GreenfootImage ("../images/vulnerableghost.gif");

    private static final int UP = 0;
    private static final int RIGHT = 1;
    private static final int DOWN = 2;
    private static final int LEFT = 3;
    private static final int DIRECTIONS = 4;
    
    private static boolean isVulnerable = false;

    // Keep track of the original image assigned to the ghost, so it can be restored
    // when the ghost is no longer vulnerable.
    public Ghost(int ghostNumber)
    {
        // Make ghosts different colors
        if (ghostNumber%NUM_GHOSTS==0)
        {
            setImage(TURQUOISE_GHOST);
        }
        else if (ghostNumber%NUM_GHOSTS==1)
        {
            setImage(PINK_GHOST);
        }
        else if (ghostNumber%NUM_GHOSTS==2)
        {
            setImage(ORANGE_GHOST);
        }
    }  // end Ghost constructor
    
    /**
     * Act - do whatever the Ghost wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        int x = getX();
        int y = getY();
        
        int dx = 0;
        int dy = 0;
        
        int direction = Greenfoot.getRandomNumber(DIRECTIONS);
                    
        if (Ghost.isVulnerable)
        {
            setImage(Ghost.VULNERABLE_GHOST);
        }
        else
        {
            setImage(Ghost.TURQUOISE_GHOST);
        }
        
        //
        if (direction == UP)
        {
            dy = -1;  // up
        }
        else if (direction == RIGHT)
        {
            dx = 1;  // right
        }
        else if (direction == DOWN)
        {
            dy = 1;  // down
        }
        else if (direction == LEFT)
        {
            dx = -1;  // left
        }
        
        List<Actor> allObjects = getObjectsAtOffset(dx,dy,Actor.class);
        if (allObjects.size()==0)
        {
            setLocation(x+dx, y+dy);
        }
        
        for(int index=0; index<allObjects.size(); index++)
        {
            // Do stuff here
            Actor actor = allObjects.get(index);
            if (actor instanceof Wall)
            {
                // interact with the Wall here (do nothing, cannot move through wall)
                ;
            }
            else if (actor instanceof Ghost)
            {
                // Interact with the Ghost here (can move through other ghosts so move normally)
                setLocation(x + dx, y + dy);
            }
            else if (actor instanceof Dot)
            {
                // Interact with Dot (can move through dots so move normally)
                setLocation(x+dx, y+dy);
            }
            else if (actor instanceof Energizer)
            {
                // Interact with Energizer (can move through energizers so move normally)
                setLocation(x+dx, y+dy);
            }
        }  // end for loop for objects at intended location
    }  // end act method
    
    public static void setVulnerability(boolean value)
    {
        isVulnerable = value;
    }
}
