import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.util.List;

/**
 * PacMan is the biggest and most complicated class in the PacMan game. This could change
 * if the Ghosts get smarter later. 
 * 
 * @author Deborah A. Trytten modified by DFH
 * @version 1.1
 */
public class PacMan extends Actor
{
    // These are the images which contain images of Pacman. Each image is 20 by 20.
    // When PacMan starts out, he's just a ball. When he's in action, his mouth is
    // moved around.
    private static final GreenfootImage startingImage = new GreenfootImage("../images/pacman.gif");
    private static final GreenfootImage mouthUpImage = new GreenfootImage("../images/pacmanmouthup.gif");
    
    // The rotation of the image determines the direction PacMan faces
    // These rotations are in degrees
    // The initial pacman has his mouth up
    // If that image is changed, these constants will have to be changed too
    private static final int MOUTH_DOWN = 180;
    private static final int MOUTH_RIGHT = 90;
    private static final int MOUTH_LEFT = 270;
    private static final int MOUTH_UP = 0;
    
    private static boolean isEnergized = false;
    private static int energizedTimeRemaining = 0;

    private static final int ENERGIZED_TIME = 20;

    // The constructor initializes all of the instance data (i.e., the stuff that isn't static)
    public PacMan()
    { 
        // The starting image for pacman is a solid yellow circle
        setImage(startingImage);
    }
    
    // The act method is called on each time step
    public void act() 
    {
        // The current position of Pacman
        int x = getX();
        int y = getY();
        
        // The movement to make based on user input
        int dx = 0;
        int dy = 0;
        
        // Count down the timer for energized PacMan/vulnerable Ghosts        
        if (PacMan.isEnergized)
        {
            PacMan.energizedTimeRemaining = PacMan.energizedTimeRemaining - 1;
            if (PacMan.energizedTimeRemaining == 0)
            {
                PacMan.isEnergized = false;
                // Let the Ghosts know they arent vulnerable
                Ghost.setVulnerability(false);
            }
        }
        
        // Get keyboard commands from user
        String nextKey = Greenfoot.getKey();
       
        // getKey returns null when there is no keyboard input -> do nothing (return)
        if (nextKey==null)
            return;
      
        // Make sure the image with a mouth gets displayed
        setImage(mouthUpImage);
       
        // Translate keystroke into movement choice
        if ("down".equalsIgnoreCase(nextKey))
        {
            //Go down
            setRotation(MOUTH_DOWN);
            dy = 1;
        }
        else if ("up".equalsIgnoreCase(nextKey))
        {
            // Go up
            setRotation(MOUTH_UP);
            dy = -1;
        }
        else if ("right".equalsIgnoreCase(nextKey))
        {
            // Go right
            setRotation(MOUTH_RIGHT);
            dx = 1;
        }
        else if ("left".equalsIgnoreCase(nextKey))
        {
            // Go left
            setRotation(MOUTH_LEFT);
            dx = -1;
        }
    
        List<Actor> allObjects = getObjectsAtOffset(dx,dy,Actor.class);
        if (allObjects.size()==0)
        {
            setLocation(x+dx, y+dy);
        }
        
        for(int index=0; index<allObjects.size(); index++)
        {
            // Do stuff here
            Actor actor = allObjects.get(index);
            if (actor instanceof Wall)
            {
                // interact with the Wall here (do nothing, cannot move through wall)
                ;
            }
            else if (actor instanceof Ghost)
            {
                // Interact with the Ghost here (stop game if not energized, eat ghost if energized)
                if (PacMan.isEnergized)
                {
                    getWorld().removeObject(actor);
                    setLocation(x+dx,y+dy);
                }
                else
                    Greenfoot.stop();
            }
            else if (actor instanceof Dot)
            {
                // Interact with Dot (remove it from GameBoard)
                getWorld().removeObject(actor);
                setLocation(x+dx, y+dy);
            }
            else if (actor instanceof Energizer)
            {
                // Interact with Energizer (remove it from GameBoard)
                getWorld().removeObject(actor);
                setLocation(x+dx, y+dy);
                Ghost.setVulnerability(true);
                PacMan.isEnergized = true;
                PacMan.energizedTimeRemaining = PacMan.ENERGIZED_TIME;
            }
        }

    } // end act
    
} // end PacMan
